@@ -26,19 +26,12 @@ import operator
 
 from nuitka.PythonVersions import python_version
 
-# TODO: Monkey patching this may cause issues for compile time assessments of
-# user code. This should be solved differently.
-if python_version >= 300:
-    operator.div = operator.truediv
-    operator.idiv = operator.itruediv
-
 binary_operator_functions = {
     "Add"       : operator.add,
     "Sub"       : operator.sub,
     "Pow"       : operator.pow,
     "Mult"      : operator.mul,
-    "Div"       : operator.div,
-    "FloorDiv"  : operator.floordiv,
+    "FloorDiv"  : operator.div if python_version < 300 else operator.floordiv,
     "TrueDiv"   : operator.truediv,
     "Mod"       : operator.mod,
     "LShift"    : operator.lshift,
@@ -50,8 +43,7 @@ binary_operator_functions = {
     "ISub"      : operator.isub,
     "IPow"      : operator.ipow,
     "IMult"     : operator.imul,
-    "IDiv"      : operator.idiv,
-    "IFloorDiv" : operator.ifloordiv,
+    "IFloorDiv" : operator.idiv if python_version < 300 else operator.ifloordiv,
     "ITrueDiv"  : operator.itruediv,
     "IMod"      : operator.imod,
     "ILShift"   : operator.ilshift,
@@ -61,6 +53,11 @@ binary_operator_functions = {
     "IBitXor"   : operator.ixor,
 }
 
+# Python 2 only operator
+if python_version < 300:
+    binary_operator_functions["Div"] = operator.div  # @UndefinedVariable
+    binary_operator_functions["IDiv"] = operator.idiv  # @UndefinedVariable
+
 # Python 3.5 only operator
 if python_version >= 350:
     binary_operator_functions["MatMult"] = operator.matmul  # @UndefinedVariable
@@ -112,7 +109,7 @@ def matchException(left, right):
             raise TypeError("catching classes that do not inherit from BaseException is not allowed")
 
     # This doesn't yet work, make it error exit. and silence PyLint for now.
-    # pylint: disable=W0212
+    # pylint: disable=protected-access
     import os
     os._exit(16)
 
