@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,11 +26,15 @@ branches and make a code block out of it. But it doesn't contain any target
 language syntax.
 """
 
-from nuitka import Options
 from nuitka.__past__ import iterItems
 from nuitka.PythonVersions import python_version
 
-from . import Contexts, Emission, Helpers
+from . import Contexts, Emission
+from .AsyncgenCodes import (
+    generateMakeAsyncgenObjectCode,
+    getAsyncgenObjectCode,
+    getAsyncgenObjectDeclCode
+)
 from .AttributeCodes import (
     generateAssignmentAttributeCode,
     generateAttributeLookupCode,
@@ -86,7 +90,8 @@ from .CoroutineCodes import (
     generateAsyncNextCode,
     generateAsyncWaitCode,
     generateMakeCoroutineObjectCode,
-    getCoroutineObjectCode
+    getCoroutineObjectCode,
+    getCoroutineObjectDeclCode
 )
 from .DictCodes import (
     generateBuiltinDictCode,
@@ -118,22 +123,20 @@ from .ExpressionCodes import (
 )
 from .FrameCodes import (
     generateFramePreserveExceptionCode,
-    generateFrameRestoreExceptionCode,
-    generateStatementsFrameCode
+    generateFrameRestoreExceptionCode
 )
 from .FunctionCodes import (
     generateFunctionCallCode,
     generateFunctionCreationCode,
-    generateFunctionDeclCode,
     generateFunctionOutlineCode,
     getExportScopeCode,
     getFunctionCode,
     getFunctionDirectDecl
 )
 from .GeneratorCodes import (
-    generateGeneratorEntryCode,
     generateMakeGeneratorObjectCode,
-    getGeneratorObjectCode
+    getGeneratorObjectCode,
+    getGeneratorObjectDeclCode
 )
 from .GlobalsLocalsCodes import (
     generateBuiltinDir1Code,
@@ -142,11 +145,14 @@ from .GlobalsLocalsCodes import (
     generateBuiltinVarsCode,
     generateSetLocalsCode
 )
-from .Helpers import generateStatementCode
+from .Helpers import (
+    generateStatementSequenceCode,
+    setExpressionDispatchDict,
+    setStatementDispatchDict
+)
 from .IdCodes import generateBuiltinHashCode, generateBuiltinIdCode
 from .ImportCodes import (
     generateBuiltinImportCode,
-    generateImportModuleCode,
     generateImportModuleHardCode,
     generateImportNameCode,
     generateImportStarCode
@@ -161,7 +167,6 @@ from .IteratorCodes import (
     generateSpecialUnpackCode,
     generateUnpackCheckCode
 )
-from .LabelCodes import getStatementTrace
 from .ListCodes import (
     generateBuiltinListCode,
     generateListCreationCode,
@@ -194,6 +199,7 @@ from .ReturnCodes import (
 from .SetCodes import (
     generateBuiltinSetCode,
     generateSetCreationCode,
+    generateSetLiteralCreationCode,
     generateSetOperationAddCode,
     generateSetOperationUpdateCode
 )
@@ -222,6 +228,7 @@ from .TupleCodes import generateBuiltinTupleCode, generateTupleCreationCode
 from .VariableCodes import (
     generateAssignmentVariableCode,
     generateDelVariableCode,
+    generateLocalsDictVariableRefCode,
     generateVariableReferenceCode,
     generateVariableReleaseCode
 )
@@ -243,13 +250,18 @@ def generateFunctionBodyCode(function_body, context):
             parent   = context,
             function = function_body
         )
+    elif function_body.isExpressionClassBody():
+        function_context = Contexts.PythonFunctionDirectContext(
+            parent   = context,
+            function = function_body
+        )
     elif function_body.isExpressionCoroutineObjectBody():
         function_context = Contexts.PythonCoroutineObjectContext(
             parent   = context,
             function = function_body
         )
-    elif function_body.isExpressionClassBody():
-        function_context = Contexts.PythonFunctionDirectContext(
+    elif function_body.isExpressionAsyncgenObjectBody():
+        function_context = Contexts.PythonAsyncgenObjectContext(
             parent   = context,
             function = function_body
         )
@@ -264,24 +276,15 @@ def generateFunctionBodyCode(function_body, context):
             function = function_body
         )
 
-    function_codes = Emission.SourceCodeCollector()
-
-    generateStatementSequenceCode(
-        statement_sequence = function_body.getBody(),
-        allow_none         = True,
-        emit               = function_codes,
-        context            = function_context
-    )
-
     needs_exception_exit = function_body.mayRaiseException(BaseException)
 
     if function_body.isExpressionGeneratorObjectBody():
         function_code = getGeneratorObjectCode(
             context                = function_context,
             function_identifier    = function_identifier,
+            closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
             temp_variables         = function_body.getTempVariables(),
-            function_codes         = function_codes.codes,
             needs_exception_exit   = needs_exception_exit,
             needs_generator_return = function_body.needsGeneratorReturnExit()
         )
@@ -289,9 +292,19 @@ def generateFunctionBodyCode(function_body, context):
         function_code = getCoroutineObjectCode(
             context                = function_context,
             function_identifier    = function_identifier,
+            closure_variables      = function_body.getClosureVariables(),
+            user_variables         = function_body.getUserLocalVariables(),
+            temp_variables         = function_body.getTempVariables(),
+            needs_exception_exit   = needs_exception_exit,
+            needs_generator_return = function_body.needsGeneratorReturnExit()
+        )
+    elif function_body.isExpressionAsyncgenObjectBody():
+        function_code = getAsyncgenObjectCode(
+            context                = function_context,
+            function_identifier    = function_identifier,
+            closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
             temp_variables         = function_body.getTempVariables(),
-            function_codes         = function_codes.codes,
             needs_exception_exit   = needs_exception_exit,
             needs_generator_return = function_body.needsGeneratorReturnExit()
         )
@@ -303,7 +316,6 @@ def generateFunctionBodyCode(function_body, context):
             closure_variables    = function_body.getClosureVariables(),
             user_variables       = function_body.getUserLocalVariables(),
             temp_variables       = function_body.getTempVariables(),
-            function_codes       = function_codes.codes,
             function_doc         = function_body.getDoc(),
             needs_exception_exit = needs_exception_exit,
             file_scope           = getExportScopeCode(
@@ -320,7 +332,6 @@ def generateFunctionBodyCode(function_body, context):
             closure_variables    = function_body.getClosureVariables(),
             user_variables       = function_body.getUserLocalVariables(),
             temp_variables       = function_body.getTempVariables(),
-            function_codes       = function_codes.codes,
             function_doc         = function_body.getDoc(),
             needs_exception_exit = needs_exception_exit,
             file_scope           = getExportScopeCode(
@@ -331,63 +342,40 @@ def generateFunctionBodyCode(function_body, context):
     return function_code, function_context
 
 
-def _generateStatementSequenceCode(statement_sequence, emit, context):
-    if statement_sequence is None:
-        return
-
-    for statement in statement_sequence.getStatements():
-        if Options.shallTraceExecution():
-            source_ref = statement.getSourceReference()
-
-            statement_repr = repr(statement)
-            source_repr = source_ref.getAsString()
-
-            if python_version >= 300:
-                statement_repr = statement_repr.encode("utf8")
-                source_repr = source_repr.encode("utf8")
-
-            emit(
-                getStatementTrace(
-                    source_repr,
-                    statement_repr
-                )
-            )
-
-        # Might contain frame statement sequences as children.
-        if statement.isStatementsFrame():
-            generateStatementsFrameCode(
-                statement_sequence = statement,
-                emit               = emit,
-                context            = context
-            )
-        else:
-            generateStatementCode(
-                statement = statement,
-                emit      = emit,
-                context   = context
-            )
-
-
-def generateStatementSequenceCode(statement_sequence, emit, context,
-                                  allow_none = False):
-
-    if allow_none and statement_sequence is None:
+def _generateFunctionDeclCode(function_body, context):
+    if function_body.isExpressionGeneratorObjectBody():
+        return getGeneratorObjectDeclCode(
+            function_identifier = function_body.getCodeName(),
+        )
+    elif function_body.isExpressionCoroutineObjectBody():
+        return getCoroutineObjectDeclCode(
+            function_identifier = function_body.getCodeName(),
+        )
+    elif function_body.isExpressionAsyncgenObjectBody():
+        return getAsyncgenObjectDeclCode(
+            function_identifier = function_body.getCodeName(),
+        )
+    elif function_body.isExpressionClassBody():
+        return getFunctionDirectDecl(
+            function_identifier = function_body.getCodeName(),
+            closure_variables   = function_body.getClosureVariables(),
+            file_scope          = getExportScopeCode(
+                cross_module = False
+            ),
+            context             = context
+        )
+    elif function_body.needsDirectCall():
+        return getFunctionDirectDecl(
+            function_identifier = function_body.getCodeName(),
+            closure_variables   = function_body.getClosureVariables(),
+            file_scope          = getExportScopeCode(
+                cross_module = function_body.isCrossModuleUsed()
+            ),
+            context             = context
+        )
+    else:
         return None
 
-    assert statement_sequence.kind == "STATEMENTS_SEQUENCE", statement_sequence
-
-    statement_context = Contexts.PythonStatementCContext(context)
-
-    _generateStatementSequenceCode(
-        statement_sequence = statement_sequence,
-        emit               = emit,
-        context            = statement_context
-    )
-
-    # Complain if any temporary was not dealt with yet.
-    assert not statement_context.getCleanupTempnames(), \
-      statement_context.getCleanupTempnames()
-
 
 def prepareModuleCode(global_context, module, module_name):
     # As this not only creates all modules, but also functions, it deals
@@ -429,7 +417,7 @@ def prepareModuleCode(global_context, module, module_name):
 
         function_body_codes.append(function_code)
 
-        function_decl = generateFunctionDeclCode(
+        function_decl = _generateFunctionDeclCode(
             function_body = function_body,
             context       = function_context
 
@@ -506,7 +494,7 @@ def makeGlobalContext():
     return Contexts.PythonGlobalContext()
 
 
-Helpers.setExpressionDispatchDict(
+setExpressionDispatchDict(
     {
         "EXPRESSION_ATTRIBUTE_LOOKUP"               : generateAttributeLookupCode,
         "EXPRESSION_ATTRIBUTE_LOOKUP_SPECIAL"       : generateAttributeLookupSpecialCode,
@@ -604,7 +592,6 @@ Helpers.setExpressionDispatchDict(
         "EXPRESSION_DICT_OPERATION_NOT_IN"          : generateDictOperationInCode,
         "EXPRESSION_FUNCTION_CREATION"              : generateFunctionCreationCode,
         "EXPRESSION_FUNCTION_CALL"                  : generateFunctionCallCode,
-        "EXPRESSION_IMPORT_MODULE"                  : generateImportModuleCode,
         "EXPRESSION_IMPORT_MODULE_HARD"             : generateImportModuleHardCode,
         "EXPRESSION_IMPORT_NAME"                    : generateImportNameCode,
         "EXPRESSION_LIST_OPERATION_EXTEND"          : generateListOperationExtendCode,
@@ -612,7 +599,9 @@ Helpers.setExpressionDispatchDict(
         "EXPRESSION_MODULE_FILE_ATTRIBUTE_REF"      : generateModuleFileAttributeCode,
         "EXPRESSION_MAKE_GENERATOR_OBJECT"          : generateMakeGeneratorObjectCode,
         "EXPRESSION_MAKE_COROUTINE_OBJECT"          : generateMakeCoroutineObjectCode,
+        "EXPRESSION_MAKE_ASYNCGEN_OBJECT"           : generateMakeAsyncgenObjectCode,
         "EXPRESSION_MAKE_SET"                       : generateSetCreationCode,
+        "EXPRESSION_MAKE_SET_LITERAL"               : generateSetLiteralCreationCode,
         "EXPRESSION_MAKE_TUPLE"                     : generateTupleCreationCode,
         "EXPRESSION_MAKE_LIST"                      : generateListCreationCode,
         "EXPRESSION_MAKE_DICT"                      : generateDictionaryCreationCode,
@@ -640,10 +629,11 @@ Helpers.setExpressionDispatchDict(
         "EXPRESSION_STRING_CONCATENATION"           : generateStringContenationCode,
         "EXPRESSION_BUILTIN_FORMAT"                 : generateBuiltinFormatCode,
         "EXPRESSION_BUILTIN_ASCII"                  : generateBuiltinAsciiCode,
+        "EXPRESSION_LOCALS_VARIABLE_REF"            : generateLocalsDictVariableRefCode
     }
 )
 
-Helpers.setStatementDispatchDict(
+setStatementDispatchDict(
     {
         "STATEMENT_ASSIGNMENT_VARIABLE"      : generateAssignmentVariableCode,
         "STATEMENT_ASSIGNMENT_ATTRIBUTE"     : generateAssignmentAttributeCode,
@@ -676,7 +666,6 @@ Helpers.setStatementDispatchDict(
         "STATEMENT_EXEC"                     : generateExecCode,
         "STATEMENT_LOCALS_DICT_SYNC"         : generateLocalsDictSyncCode,
         "STATEMENT_SET_LOCALS"               : generateSetLocalsCode,
-        "STATEMENT_GENERATOR_ENTRY"          : generateGeneratorEntryCode,
         "STATEMENT_PRESERVE_FRAME_EXCEPTION" : generateFramePreserveExceptionCode,
         "STATEMENT_RESTORE_FRAME_EXCEPTION"  : generateFrameRestoreExceptionCode,
         "STATEMENT_PUBLISH_EXCEPTION"        : generateExceptionPublishCode
