@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -58,6 +58,17 @@ def getErrorExitReleaseCode(context):
     return temp_release
 
 
+def getFrameVariableTypeDescriptionCode(context):
+
+    type_description = context.getFrameVariableTypeDescription()
+
+    if type_description:
+        context.markAsNeedsFrameVariableDescription()
+        return 'type_description = "%s";' % type_description
+    else:
+        return ""
+
+
 def getErrorExitBoolCode(condition, emit, context,
                          needs_check = True, quick_exception = None):
     assert not condition.endswith(';')
@@ -72,13 +83,16 @@ def getErrorExitBoolCode(condition, emit, context,
         emit(
             indented(
                 template_error_catch_quick_exception % {
-                    "condition"        : condition,
-                    "exception_exit"   : context.getExceptionEscape(),
-                    "quick_exception"  : getExceptionIdentifier(quick_exception),
-                    "release_temps"    : indented(
+                    "condition"            : condition,
+                    "exception_exit"       : context.getExceptionEscape(),
+                    "quick_exception"      : getExceptionIdentifier(quick_exception),
+                    "release_temps"        : indented(
                         getErrorExitReleaseCode(context)
                     ),
-                    "line_number_code" : indented(
+                    "var_description_code" : indented(
+                        getFrameVariableTypeDescriptionCode(context),
+                    ),
+                    "line_number_code"     : indented(
                         getErrorLineNumberUpdateCode(context)
                     )
                 },
@@ -94,6 +108,9 @@ def getErrorExitBoolCode(condition, emit, context,
                     "release_temps"    : indented(
                         getErrorExitReleaseCode(context)
                     ),
+                    "var_description_code": indented(
+                        getFrameVariableTypeDescriptionCode(context),
+                    ),
                     "line_number_code" : indented(
                         getErrorLineNumberUpdateCode(context)
                     )
@@ -120,7 +137,7 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
 
     context.markAsNeedsExceptionVariables()
 
-    if len(args) == 1:
+    if len(args) == 1 and type(args[0]) is str:
         from .ConstantCodes import getModuleConstantCode
 
         set_exception = [
@@ -132,7 +149,15 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
             "exception_tb = NULL;"
         ]
     else:
-        assert False, args
+        set_exception = [
+            "exception_type = %s;" % exception,
+            "Py_INCREF( exception_type );",
+            "exception_value = Py%s_FromFormat( %s );" % (
+                "String" if python_version < 300 else "Unicode",
+                ", ".join( '"%s"' % arg for arg in args )
+            ),
+            "exception_tb = NULL;"
+        ]
 
 
     if python_version >= 300:
@@ -161,12 +186,16 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
             "release_temps"    : indented(
                 getErrorExitReleaseCode(context)
             ),
+            "var_description_code" : indented(
+                getFrameVariableTypeDescriptionCode(context),
+            ),
             "line_number_code" : indented(
                 getErrorLineNumberUpdateCode(context)
             )
         }
     )
 
+
 def getErrorVariableDeclarations():
     return (
         "PyObject *exception_type = NULL, *exception_value = NULL;",
@@ -272,13 +301,17 @@ def getMustNotGetHereCode(reason, context, emit):
         emit("return;")
     elif provider.isExpressionCoroutineObjectBody():
         emit("return;")
-    elif provider.isExpressionFunctionBody():
-        emit("return NULL;")
+    elif provider.isExpressionAsyncgenObjectBody():
+        emit("return;")
     elif provider.isCompiledPythonModule():
         emit("return MOD_RETURN_VALUE( NULL );")
     else:
-        emit("return;")
+        emit("return NULL;")
 
 
 def getAssertionCode(check, emit):
     emit("assert( %s );" % check)
+
+
+def getCheckObjectCode(check_name, emit):
+    emit("CHECK_OBJECT( %s );" % check_name)
