@@ -20,6 +20,7 @@
 This also includes writing back to locals for exec statements.
 """
 
+from nuitka.codegen.VariableCodes import getLocalVariableCodeType
 from nuitka.PythonVersions import python_version
 
 from .ErrorCodes import getErrorExitBoolCode
@@ -32,27 +33,25 @@ from .templates.CodeTemplatesVariables import (
     template_update_locals_dict_value,
     template_update_locals_mapping_value
 )
-from .VariableCodes import (
-    getLocalVariableObjectAccessCode,
-    getVariableAssignmentCode
-)
+from .VariableCodes import getVariableAssignmentCode
 
 
 def generateBuiltinLocalsCode(to_name, expression, emit, context):
     provider = expression.getParentVariableProvider()
 
     getLoadLocalsCode(
-        to_name  = to_name,
-        provider = provider,
-        mode     = provider.getLocalsMode(),
-        emit     = emit,
-        context  = context
+        to_name   = to_name,
+        variables = expression.getVariableVersions(),
+        provider  = provider,
+        mode      = provider.getLocalsMode(),
+        emit      = emit,
+        context   = context
     )
 
 
 def generateBuiltinGlobalsCode(to_name, expression, emit, context):
     # Functions used for generation all accept expression, but this one does
-    # not use it. pylint: disable=W0613
+    # not use it. pylint: disable=unused-argument
 
     getLoadGlobalsCode(
         to_name = to_name,
@@ -85,19 +84,17 @@ def _getLocalVariableList(provider):
         for variable in
         provider.getVariables()
         if not variable.isModuleVariable()
-        if not variable.isMaybeLocalVariable()
         if (include_closure or variable.getOwner() is provider)
     ]
 
 
-def _getVariableDictUpdateCode(target_name, variable, initial, is_dict, emit, context):
+def _getVariableDictUpdateCode(target_name, variable, version, initial, is_dict, emit, context):
     # TODO: Variable could known to be set here, get a hand at that
     # information.
 
-    access_code = getLocalVariableObjectAccessCode(
-        variable = variable,
-        context  = context
-    )
+    variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable, version)
+
+    access_code = variable_c_type.getLocalVariableObjectAccessCode(variable_code_name)
 
     if is_dict:
         if initial:
@@ -140,14 +137,23 @@ def _getVariableDictUpdateCode(target_name, variable, initial, is_dict, emit, co
         )
 
 
-def getLoadLocalsCode(to_name, provider, mode, emit, context):
+def getLoadLocalsCode(to_name, variables, provider, mode, emit, context):
+
+    def _sorted(variables):
+        all_variables = list(context.getOwner().getVariables())
+
+        return sorted(
+            variables,
+            key = lambda variable_desc: (
+                all_variables.index(variable_desc[0]),
+                variable_desc[1]
+            )
+        )
+
     if provider.isCompiledPythonModule():
         # Optimization will have made this "globals".
         assert False, provider
     elif not context.hasLocalsDict():
-        local_list = _getLocalVariableList(
-            provider = provider,
-        )
 
         # TODO: Use DictCodes ?
         emit(
@@ -158,10 +164,11 @@ def getLoadLocalsCode(to_name, provider, mode, emit, context):
 
         context.addCleanupTempName(to_name)
 
-        for local_var in local_list:
+        for variable, version in _sorted(variables):
             _getVariableDictUpdateCode(
                 target_name = to_name,
-                variable    = local_var,
+                variable    = variable,
+                version     = version,
                 is_dict     = True,
                 initial     = True,
                 emit        = emit,
@@ -177,10 +184,6 @@ def getLoadLocalsCode(to_name, provider, mode, emit, context):
 
             context.addCleanupTempName(to_name)
         elif mode == "updated":
-            local_list = _getLocalVariableList(
-                provider = provider
-            )
-
             emit(
                 """\
 %s = locals_dict;
@@ -189,10 +192,17 @@ Py_INCREF( locals_dict );""" % (
                 )
             )
 
-            for local_var in local_list:
+            variables = [
+                (variable, variable_version)
+                for variable, variable_version in
+                variables
+            ]
+
+            for local_var, version in _sorted(variables):
                 _getVariableDictUpdateCode(
                     target_name = to_name,
                     variable    = local_var,
+                    version     = version,
                     is_dict     = python_version < 300 or \
                                   not context.getFunction().isExpressionClassBody(),
                     initial     = False,
@@ -234,12 +244,11 @@ locals_dict = %s;""" % (
         context.removeCleanupTempName(new_locals_name)
 
 
-def getStoreLocalsCode(locals_name, provider, emit, context):
+def getStoreLocalsCode(locals_name, variables, provider, emit, context):
     assert not provider.isCompiledPythonModule()
 
-    for variable in provider.getVariables():
-        if not variable.isModuleVariable() and \
-           not variable.isMaybeLocalVariable():
+    for variable, version in variables:
+        if not variable.isModuleVariable():
             key_name = context.getConstantCode(
                 constant = variable.getName()
             )
@@ -269,6 +278,7 @@ def getStoreLocalsCode(locals_name, provider, emit, context):
             context.addCleanupTempName(value_name)
             getVariableAssignmentCode(
                 variable      = variable,
+                version       = version,
                 tmp_name      = value_name,
                 needs_release = None, # TODO: Could be known maybe.
                 in_place      = False,
