@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,14 +19,10 @@
 
 """
 
-from nuitka import Options
+from nuitka.Version import getNuitkaVersion, getNuitkaVersionYear
 
 from .CodeObjectCodes import getCodeObjectsDeclCode, getCodeObjectsInitCode
-from .ConstantCodes import (
-    allocateNestedConstants,
-    getConstantCode,
-    getConstantInitCodes
-)
+from .ConstantCodes import allocateNestedConstants, getConstantInitCodes
 from .ErrorCodes import (
     getErrorVariableDeclarations,
     getExceptionKeeperVariableNames,
@@ -50,13 +46,15 @@ def getModuleValues(context, module_name, module_identifier, codes,
                     function_decl_codes, function_body_codes, temp_variables,
                     is_main_module, is_internal_module):
     # For the module code, lots of arguments and attributes come together.
-    # pylint: disable=R0914
+    # pylint: disable=too-many-locals
 
     # Temporary variable initializations
     local_var_inits = [
         getLocalVariableInitCode(
-            context  = context,
-            variable = variable
+            context   = context,
+            variable  = variable,
+            version   = 0,
+            init_from = None
         )
         for variable in
         temp_variables
@@ -86,6 +84,9 @@ def getModuleValues(context, module_name, module_identifier, codes,
 
     local_var_inits += context.getFrameDeclarations()
 
+    if context.needsFrameVariableTypeDescription():
+        local_var_inits.append("char *type_description;")
+
     if context.needsExceptionVariables():
         module_exit = template_module_exception_exit
     else:
@@ -93,8 +94,7 @@ def getModuleValues(context, module_name, module_identifier, codes,
 
     module_body_template_values = {
         "module_name"              : module_name,
-        "module_name_obj"          : getConstantCode(
-            context  = context,
+        "module_name_obj"          : context.getConstantCode(
             constant = module_name
         ),
         "is_main_module"           : 1 if is_main_module else 0,
@@ -128,8 +128,8 @@ def getModuleValues(context, module_name, module_identifier, codes,
 def getModuleCode(module_context, template_values):
     header = template_global_copyright % {
         "name"    : module_context.getName(),
-        "version" : Options.getVersion(),
-        "year"    : Options.getYear()
+        "version" : getNuitkaVersion(),
+        "year"    : getNuitkaVersionYear()
     }
 
     decls, inits, checks = getConstantInitCodes(module_context)
@@ -157,7 +157,7 @@ def getModuleCode(module_context, template_values):
 
 def generateModuleFileAttributeCode(to_name, expression, emit, context):
     # The expression doesn't really matter, but it is part of the API for
-    # the expression registry, pylint: disable=W0613
+    # the expression registry, pylint: disable=unused-argument
 
     emit(
         "%s = module_filename_obj;" % (
