@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -130,9 +130,9 @@ if ( %(result)s == true )
 }
 """
 
-# TODO: Storage will not be NULL. What is this used for.
 template_del_shared_intolerant = """\
-%(result)s = %(identifier)s != NULL && PyCell_GET( %(identifier)s ) != NULL;
+assert( %(identifier)s != NULL );
+%(result)s = PyCell_GET( %(identifier)s ) != NULL;
 if ( %(result)s == true )
 {
     Py_DECREF( PyCell_GET( %(identifier)s ) );
@@ -152,6 +152,8 @@ PyCell_SET( %(identifier)s, NULL );
 """
 
 
+# TODO: We could know, if we could loop, and only set the
+# variable to NULL then, using a different template.
 template_release_unclear = """\
 Py_XDECREF( %(identifier)s );
 %(identifier)s = NULL;
@@ -163,13 +165,6 @@ Py_DECREF( %(identifier)s );
 %(identifier)s = NULL;
 """
 
-# TODO: Maybe storage is never NULL.
-template_check_shared = """\
-( %(identifier)s != NULL && PyCell_GET( %(identifier)s ) != NULL )"""
-
-template_check_local = """\
-( %(identifier)s != NULL )"""
-
 # TODO: Storage will not be NULL.
 template_read_shared_unclear = """\
 if ( %(identifier)s == NULL )
@@ -189,6 +184,9 @@ template_read_shared_known = """\
 # For module variable values, need to lookup in module dictionary or in
 # built-in dictionary.
 
+# TODO: Only provide fallback for known actually possible values. Do this
+# by keeping track of things that were added by "site.py" mechanisms. Then
+# we can avoid the second call entirely for most cases.
 template_read_mvar_unclear = """\
 %(tmp_name)s = GET_STRING_DICT_VALUE( moduledict_%(module_identifier)s, (Nuitka_StringObject *)%(var_name)s );
 
@@ -212,5 +210,51 @@ template_del_global_unclear = """\
 if ( %(res_name)s == -1 ) CLEAR_ERROR_OCCURRED();
 """
 
+template_update_locals_dict_value = """\
+DICT_SYNC_FROM_VARIABLE(
+    %(dict_name)s,
+    %(var_name)s,
+    %(access_code)s
+);
+"""
+
+template_set_locals_dict_value = """\
+if ( %(access_code)s )
+{
+    int res = PyDict_SetItem(
+        %(dict_name)s,
+        %(var_name)s,
+        %(access_code)s
+    );
+
+    assert( res == 0 );
+}
+"""
+
+template_update_locals_mapping_value = """\
+%(tmp_name)s = MAPPING_SYNC_FROM_VARIABLE( %(mapping_name)s, %(var_name)s, %(access_code)s );
+"""
+
+template_set_locals_mapping_value = """\
+if %(check_code)s
+{
+    %(tmp_name)s = SET_SUBSCRIPT(
+        %(mapping_name)s,
+        %(var_name)s,
+        %(access_code)s
+    );
+}
+"""
+
+
+# TODO: Unused now.
+template_assign_from_frame_locals = """\
+if ( %(frame_identifier)s->f_locals == NULL )
+{
+    %(frame_identifier)s->f_locals = PyDict_New();
+}
+%(to_name)s = %(frame_identifier)s->f_locals;
+"""
+
 from . import TemplateDebugWrapper # isort:skip
 TemplateDebugWrapper.checkDebug(globals())
