@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,7 @@ check it many times.
 """
 from nuitka.tree import Operations
 
-from .FinalizeClosureTaking import FinalizeClassClosure, FinalizeClosureTaking
+from .FinalizeClosureTaking import FinalizeClassClosure
 from .FinalizeMarkups import FinalizeMarkups
 
 
@@ -38,7 +38,3 @@ def prepareCodeGeneration(tree):
     for function in tree.getUsedFunctions():
         if function.hasFlag("has_super"):
             Operations.visitFunction(function, visitor)
-
-    visitor = FinalizeClosureTaking()
-    for function in tree.getUsedFunctions():
-        Operations.visitFunction(function, visitor)
