@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,20 +21,21 @@ Useful to getting an idea of what the internal representation of Nuitka is about
 code.
 """
 
+import os
 import sys
 
-from PyQt5 import QtCore  # @UnresolvedImport pylint: disable=F0401,I0021
-from PyQt5 import QtGui  # @UnresolvedImport pylint: disable=F0401,I0021
-from PyQt5 import uic  # @UnresolvedImport pylint: disable=F0401,I0021
-
 from nuitka import SourceCodeReferences
-from nuitka.utils import Utils
 
+from PyQt5 import ( # isort:skip pylint: disable=import-error,useless-suppression
+    QtCore,   # @UnresolvedImport
+    QtGui,   # @UnresolvedImport
+    uic
+)
 
-# The API requires a signature, sometimes we don't use it, pylint: disable=R0201
-# Also using private stuff from classes, probably ok, pylint: disable=W0212
+# The API requires a signature, sometimes we don't use it, pylint: disable=no-self-use
+# Also using private stuff from classes, probably ok, pylint: disable=protected-access
 
-class NodeTreeModelItem:
+class NodeTreeModelItem(object):
     def __init__(self, node, parent = None):
         self.parent_treeitem = parent
         self.node = node
@@ -200,8 +201,12 @@ class InspectNodeTreeDialog(QtGui.QDialog):
     def __init__(self, *args):
         QtGui.QDialog.__init__(self, *args)
 
-        ui_dir = Utils.dirname(__file__)
-        ui_filename = Utils.joinpath(ui_dir, "dialogs", "InspectPythonTree.ui")
+        ui_dir = os.path.dirname(__file__)
+        ui_filename = os.path.join(
+            ui_dir,
+            "dialogs",
+            "InspectPythonTree.ui"
+        )
 
         uic.loadUi(ui_filename, self)
 
