@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,16 +20,17 @@
 """
 
 import glob
+import marshal
+import os
 import sys
 from logging import debug, info, warning
 
-import marshal
 from nuitka import ModuleRegistry, Options
 from nuitka.importing import ImportCache, Importing, StandardLibrary
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.tree.SourceReading import readSourceCodeFromFilename
-from nuitka.utils import Utils
+from nuitka.utils.FileOperations import listDir, relpath
 
 
 def logRecursion(*args):
@@ -65,13 +66,15 @@ def recurseTo(module_package, module_filename, module_relpath, module_kind,
 
             if module_kind == "py" and source_filename is not None:
                 try:
+                    source_code = readSourceCodeFromFilename(
+                        module_name     = module.getFullName(),
+                        source_filename = source_filename
+                    )
+
                     Building.createModuleTree(
                         module      = module,
                         source_ref  = source_ref,
-                        source_code = readSourceCodeFromFilename(
-                            module_name     = module.getFullName(),
-                            source_filename = source_filename
-                        ),
+                        source_code = source_code,
                         is_main     = False
                     )
                 except (SyntaxError, IndentationError) as e:
@@ -98,14 +101,13 @@ Cannot recurse to import module '%s' (%s) because code is too complex.""",
                             module_filename,
                         )
 
-
                         if Options.isStandaloneMode():
                             module = makeUncompiledPythonModule(
                                 module_name   = module.getFullName(),
                                 filename      = module_filename,
                                 bytecode      = marshal.dumps(
                                     compile(
-                                        readSourceCodeFromFilename(module.getFullName(), module_filename),
+                                        source_code,
                                         module_filename,
                                         "exec"
                                     )
@@ -138,7 +140,7 @@ Cannot recurse to import module '%s' (%s) because code is too complex.""",
 
 def decideRecursion(module_filename, module_name, module_package, module_kind):
     # Many branches, which make decisions immediately, by returning
-    # pylint: disable=R0911,R0912
+    # pylint: disable=too-many-branches,too-many-return-statements
     Plugins.onModuleEncounter(
         module_filename,
         module_name,
@@ -215,18 +217,18 @@ def decideRecursion(module_filename, module_name, module_package, module_kind):
 
 
 def considerFilename(module_filename):
-    module_filename = Utils.normpath(module_filename)
+    module_filename = os.path.normpath(module_filename)
 
-    if Utils.isDir(module_filename):
-        module_filename = Utils.abspath(module_filename)
+    if os.path.isdir(module_filename):
+        module_filename = os.path.abspath(module_filename)
 
-        module_name = Utils.basename(module_filename)
-        module_relpath = Utils.relpath(module_filename)
+        module_name = os.path.basename(module_filename)
+        module_relpath = relpath(module_filename)
 
         return module_filename, module_relpath, module_name
     elif module_filename.endswith(".py"):
-        module_name = Utils.basename(module_filename)[:-3]
-        module_relpath = Utils.relpath(module_filename)
+        module_name = os.path.basename(module_filename)[:-3]
+        module_relpath = relpath(module_filename)
 
         return module_filename, module_relpath, module_name
     else:
@@ -234,16 +236,16 @@ def considerFilename(module_filename):
 
 
 def isSameModulePath(path1, path2):
-    if Utils.basename(path1) == "__init__.py":
-        path1 = Utils.dirname(path1)
-    if Utils.basename(path2) == "__init__.py":
-        path2 = Utils.dirname(path2)
+    if os.path.basename(path1) == "__init__.py":
+        path1 = os.path.dirname(path1)
+    if os.path.basename(path2) == "__init__.py":
+        path2 = os.path.dirname(path2)
 
-    return Utils.abspath(path1) == Utils.abspath(path2)
+    return os.path.abspath(path1) == os.path.abspath(path2)
 
 
 def _checkPluginPath(plugin_filename, module_package):
-    # Many branches, for the decision is very complex, pylint: disable=R0912
+    # Many branches, for the decision is very complex, pylint: disable=too-many-branches
 
     debug(
         "Checking detail plug-in path '%s' '%s':",
@@ -262,7 +264,7 @@ def _checkPluginPath(plugin_filename, module_package):
         )
 
         if decision:
-            module_relpath = Utils.relpath(plugin_filename)
+            module_relpath = relpath(plugin_filename)
 
             module, is_added = recurseTo(
                 module_filename = plugin_filename,
@@ -301,7 +303,7 @@ def _checkPluginPath(plugin_filename, module_package):
                 if module.isCompiledPythonPackage():
                     package_filename = module.getFilename()
 
-                    if Utils.isDir(package_filename):
+                    if os.path.isdir(package_filename):
                         # Must be a namespace package.
                         assert python_version >= 330
 
@@ -310,7 +312,7 @@ def _checkPluginPath(plugin_filename, module_package):
                         # Only include it, if it contains actual modules, which will
                         # recurse to this one and find it again.
                     else:
-                        package_dir = Utils.dirname(package_filename)
+                        package_dir = os.path.dirname(package_filename)
 
                         # Real packages will always be included.
                         ModuleRegistry.addRootModule(module)
@@ -320,7 +322,7 @@ def _checkPluginPath(plugin_filename, module_package):
                         package_dir
                     )
 
-                    for sub_path, sub_filename in Utils.listDir(package_dir):
+                    for sub_path, sub_filename in listDir(package_dir):
                         if sub_filename in ("__init__.py", "__pycache__"):
                             continue
 
@@ -350,11 +352,11 @@ def checkPluginPath(plugin_filename, module_package):
 
     if plugin_info is not None:
         # File or package makes a difference, handle that
-        if Utils.isFile(plugin_info[0]) or \
+        if os.path.isfile(plugin_info[0]) or \
            Importing.isPackageDir(plugin_info[0]):
             _checkPluginPath(plugin_filename, module_package)
-        elif Utils.isDir(plugin_info[0]):
-            for sub_path, sub_filename in Utils.listDir(plugin_info[0]):
+        elif os.path.isdir(plugin_info[0]):
+            for sub_path, sub_filename in listDir(plugin_info[0]):
                 assert sub_filename != "__init__.py"
 
                 if Importing.isPackageDir(sub_path) or \
@@ -372,7 +374,7 @@ def checkPluginFilenamePattern(pattern):
         pattern,
     )
 
-    if Utils.isDir(pattern):
+    if os.path.isdir(pattern):
         sys.exit("Error, pattern cannot be a directory name.")
 
     found = False
@@ -381,7 +383,7 @@ def checkPluginFilenamePattern(pattern):
         if filename.endswith(".pyc"):
             continue
 
-        if not Utils.isFile(filename):
+        if not os.path.isfile(filename):
             continue
 
         found = True
