@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,7 +21,7 @@
 
 from nuitka import PythonOperators
 
-from .NodeBases import ExpressionChildrenHavingBase
+from .ExpressionBases import ExpressionChildrenHavingBase
 from .NodeMakingHelpers import (
     makeComparisonNode,
     makeConstantReplacementNode,
@@ -142,7 +142,7 @@ class ExpressionComparisonIsIsNotBase(ExpressionComparison):
         return {}
 
     def isExpressionComparison(self):
-        # Virtual method, pylint: disable=R0201
+        # Virtual method, pylint: disable=no-self-use
         return True
 
     def mayRaiseException(self, exception_type):
@@ -254,7 +254,7 @@ class ExpressionComparisonExceptionMatch(ExpressionComparison):
         return {}
 
     def isExpressionComparison(self):
-        # Virtual method, pylint: disable=R0201
+        # Virtual method, pylint: disable=no-self-use
         return True
 
     def getSimulator(self):
@@ -279,7 +279,7 @@ class ExpressionComparisonInNotInBase(ExpressionComparison):
         return {}
 
     def isExpressionComparison(self):
-        # Virtual method, pylint: disable=R0201
+        # Virtual method, pylint: disable=no-self-use
         return True
 
     def mayRaiseException(self, exception_type):
