@@ -25,12 +25,13 @@ whose implementation lives here. The creation itself also lives here.
 from nuitka.PythonVersions import python_version
 
 from .Checkers import checkStatementsSequenceOrNone
+from .ExpressionBases import ExpressionChildrenHavingBase
 from .FunctionNodes import ExpressionFunctionBodyBase
 from .IndicatorMixins import (
-    MarkLocalsDictIndicator,
-    MarkUnoptimizedFunctionIndicator
+    MarkLocalsDictIndicatorMixin,
+    MarkUnoptimizedFunctionIndicatorMixin
 )
-from .NodeBases import ChildrenHavingMixin, ExpressionChildrenHavingBase
+from .NodeBases import ChildrenHavingMixin
 from .ReturnNodes import StatementReturn
 
 
@@ -56,6 +57,8 @@ class ExpressionMakeGeneratorObject(ExpressionChildrenHavingBase):
 
         self.code_object = code_object
 
+        self.variable_closure_traces = None
+
     def getDetails(self):
         return {
             "code_object" : self.code_object
@@ -65,6 +68,14 @@ class ExpressionMakeGeneratorObject(ExpressionChildrenHavingBase):
         return self.code_object
 
     def computeExpression(self, trace_collection):
+        self.variable_closure_traces = []
+
+        for closure_variable in self.getGeneratorRef().getFunctionBody().getClosureVariables():
+            trace = trace_collection.getVariableCurrentTrace(closure_variable)
+            trace.addClosureUsage()
+
+            self.variable_closure_traces.append(trace)
+
         # TODO: Generator body may know something too.
         return self, None, None
 
@@ -74,10 +85,17 @@ class ExpressionMakeGeneratorObject(ExpressionChildrenHavingBase):
     def mayHaveSideEffects(self):
         return False
 
+    def getClosureVariableVersions(self):
+        return [
+            (trace.getVariable(), trace.getVersion())
+            for trace in self.variable_closure_traces
+        ]
 
-class ExpressionGeneratorObjectBody(ExpressionFunctionBodyBase,
-                                    MarkLocalsDictIndicator,
-                                    MarkUnoptimizedFunctionIndicator):
+
+
+class ExpressionGeneratorObjectBody(MarkLocalsDictIndicatorMixin,
+                                    MarkUnoptimizedFunctionIndicatorMixin,
+                                    ExpressionFunctionBodyBase):
     # We really want these many ancestors, as per design, we add properties via
     # base class mix-ins a lot, pylint: disable=R0901
     kind = "EXPRESSION_GENERATOR_OBJECT_BODY"
@@ -105,9 +123,9 @@ class ExpressionGeneratorObjectBody(ExpressionFunctionBodyBase,
             source_ref  = source_ref
         )
 
-        MarkLocalsDictIndicator.__init__(self)
+        MarkLocalsDictIndicatorMixin.__init__(self)
 
-        MarkUnoptimizedFunctionIndicator.__init__(self)
+        MarkUnoptimizedFunctionIndicatorMixin.__init__(self, flags)
 
         self.needs_generator_return_exit = False
 
