@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,7 +23,8 @@ own anything by themselves. It's just a way of having try/finally for the
 expressions, or multiple returns, without running in a too different context.
 """
 
-from .NodeBases import ChildrenHavingMixin, ExpressionChildrenHavingBase
+from .ExpressionBases import ExpressionChildrenHavingBase
+from .NodeBases import ChildrenHavingMixin
 
 
 class ExpressionOutlineBody(ExpressionChildrenHavingBase):
