@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -49,10 +49,12 @@ catching and passing in exceptions raised.
 
 """
 
+import os
 import sys
+from logging import info, warning
 
-from nuitka import Options, SourceCodeReferences, Tracing
-from nuitka.__past__ import long, unicode  # pylint: disable=W0622
+from nuitka import Options, SourceCodeReferences
+from nuitka.__past__ import long, unicode  # pylint: disable=redefined-builtin
 from nuitka.importing import Importing
 from nuitka.importing.ImportCache import addImportedModule
 from nuitka.importing.PreloadedPackages import getPthImportedPackages
@@ -61,6 +63,11 @@ from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable
 )
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
+from nuitka.nodes.BuiltinFormatNodes import (
+    ExpressionBuiltinAscii,
+    ExpressionBuiltinFormat
+)
+from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinStr
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
@@ -73,10 +80,6 @@ from nuitka.nodes.ConstantRefNodes import (
 from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
 from nuitka.nodes.ExceptionNodes import StatementRaiseException
 from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
-from nuitka.nodes.ImportNodes import (
-    ExpressionImportModule,
-    ExpressionImportName
-)
 from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
 from nuitka.nodes.ModuleNodes import (
     CompiledPythonModule,
@@ -86,32 +89,32 @@ from nuitka.nodes.ModuleNodes import (
     PythonShlibModule
 )
 from nuitka.nodes.OperatorNodes import (
-    ExpressionOperationBinary,
-    ExpressionOperationUnary
+    ExpressionOperationUnary,
+    makeBinaryOperationNode
 )
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
+from nuitka.nodes.StringConcatenationNodes import ExpressionStringConcatenation
 from nuitka.nodes.VariableRefNodes import ExpressionVariableRef
+from nuitka.Options import shallWarnUnusualCode
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
-from nuitka.tree.ReformulationForLoopStatements import (
-    buildAsyncForLoopNode,
-    buildForLoopNode
-)
-from nuitka.tree.ReformulationWhileLoopStatements import buildWhileLoopNode
-from nuitka.utils import MemoryUsage, Utils
+from nuitka.tree.ReformulationImportStatements import getFutureSpec
+from nuitka.utils import MemoryUsage
+from nuitka.utils.FileOperations import splitPath
 
 from . import SyntaxErrors
 from .Helpers import (
     buildNode,
+    buildNodeList,
     buildStatementsNode,
     extractDocFromBody,
     getBuildContext,
     getKind,
+    makeAbsoluteImportNode,
     makeModuleFrame,
     makeStatementsSequence,
     makeStatementsSequenceFromStatement,
-    makeStatementsSequenceOrStatement,
     mangleName,
     mergeStatements,
     parseSourceCodeToAst,
@@ -119,6 +122,7 @@ from .Helpers import (
 )
 from .ReformulationAssertStatements import buildAssertNode
 from .ReformulationAssignmentStatements import (
+    buildAnnAssignNode,
     buildAssignNode,
     buildDeleteNode,
     buildInplaceAssignNode
@@ -135,13 +139,20 @@ from .ReformulationContractionExpressions import (
 )
 from .ReformulationDictionaryCreation import buildDictionaryNode
 from .ReformulationExecStatements import buildExecNode
+from .ReformulationForLoopStatements import (
+    buildAsyncForLoopNode,
+    buildForLoopNode
+)
 from .ReformulationFunctionStatements import (
     buildAsyncFunctionNode,
     buildFunctionNode
 )
 from .ReformulationImportStatements import (
     buildImportFromNode,
-    checkFutureImportsOnlyAtStart
+    buildImportModulesNode,
+    checkFutureImportsOnlyAtStart,
+    popFutureSpec,
+    pushFutureSpec
 )
 from .ReformulationLambdaExpressions import buildLambdaNode
 from .ReformulationNamespacePackages import (
@@ -153,9 +164,13 @@ from .ReformulationSequenceCreation import buildSequenceCreationNode
 from .ReformulationSubscriptExpressions import buildSubscriptNode
 from .ReformulationTryExceptStatements import buildTryExceptionNode
 from .ReformulationTryFinallyStatements import buildTryFinallyNode
+from .ReformulationWhileLoopStatements import buildWhileLoopNode
 from .ReformulationWithStatements import buildAsyncWithNode, buildWithNode
 from .ReformulationYieldExpressions import buildYieldFromNode, buildYieldNode
-from .SourceReading import readSourceCodeFromFilename
+from .SourceReading import (
+    checkPythonVersionFromCode,
+    readSourceCodeFromFilename
+)
 from .VariableClosure import completeVariableClosures
 
 
@@ -195,7 +210,7 @@ def buildConditionNode(provider, node, source_ref):
     )
 
 
-def _buildTryFinallyNode(provider, node, source_ref):
+def buildTryFinallyNode2(provider, node, source_ref):
     # Try/finally node statements of old style.
 
     return buildTryFinallyNode(
@@ -217,7 +232,7 @@ def buildTryNode(provider, node, source_ref):
 
     # Shortcut missing try/finally.
     if not node.handlers:
-        return _buildTryFinallyNode(provider, node, source_ref)
+        return buildTryFinallyNode2(provider, node, source_ref)
 
     if not node.finalbody:
         return buildTryExceptionNode(
@@ -289,86 +304,17 @@ def buildRaiseNode(provider, node, source_ref):
 
     return result
 
-def buildImportModulesNode(provider, node, source_ref):
-    # Import modules statement. As described in the developer manual, these
-    # statements can be treated as several ones.
-
-    import_names   = [
-        ( import_desc.name, import_desc.asname )
-        for import_desc in
-        node.names
-    ]
-
-    import_nodes = []
-
-    for import_desc in import_names:
-        module_name, local_name = import_desc
-
-        module_topname = module_name.split('.')[0]
-
-        # Note: The "level" of import is influenced by the future absolute
-        # imports.
-        level = 0 if source_ref.getFutureSpec().isAbsoluteImport() else -1
-
-        if local_name:
-            # If is gets a local name, the real name must be used as a
-            # temporary value only, being looked up recursively.
-
-            import_node = ExpressionImportModule(
-                module_name = module_name,
-                import_list = None,
-                level       = level,
-                source_ref  = source_ref
-            )
-
-            for import_name in module_name.split('.')[1:]:
-                import_node = ExpressionImportName(
-                    module      = import_node,
-                    import_name = import_name,
-                    source_ref  = source_ref
-                )
-        else:
-            import_node = ExpressionImportModule(
-                module_name = module_name,
-                import_list = None,
-                level       = level,
-                source_ref  = source_ref
-            )
-
-        # If a name was given, use the one provided, otherwise the import gives
-        # the top level package name given for assignment of the imported
-        # module.
-
-        import_nodes.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = mangleName(
-                        local_name
-                          if local_name is not None else
-                        module_topname,
-                        provider
-                    ),
-                    source_ref    = source_ref
-                ),
-                source       = import_node,
-                source_ref   = source_ref
-            )
-        )
-
-    # Note: Each import is sequential. It will potentially succeed, and the
-    # failure of a later one is not changing that one bit . We can therefore
-    # have a sequence of imports that only import one thing therefore.
-    return makeStatementsSequenceOrStatement(
-        statements = import_nodes,
-        source_ref = source_ref
-    )
-
 
 def handleGlobalDeclarationNode(provider, node, source_ref):
 
-    # On the module level, there is nothing to do. TODO: Probably a warning
-    # would be warranted.
+    # On the module level, there is nothing to do.
     if provider.isCompiledPythonModule():
+        if shallWarnUnusualCode():
+            warning(
+                "%s: Using 'global' statement on module level has no effect.",
+                source_ref.getAsString(),
+            )
+
         return None
 
     # Need to catch the error of declaring a parameter variable as global
@@ -379,18 +325,13 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
         for variable_name in node.names:
             if variable_name in parameters.getParameterNames():
                 SyntaxErrors.raiseSyntaxError(
-                    reason     = "name '%s' is %s and global" % (
+                    "name '%s' is %s and global" % (
                         variable_name,
                         "local"
                           if python_version < 300 else
                         "parameter"
                     ),
-                    source_ref = (
-                        source_ref
-                          if not Options.isFullCompat() or \
-                             python_version >= 340 else
-                        provider.getSourceReference()
-                    )
+                    source_ref.atColumnNumber(node.col_offset)
                 )
 
     # The module the "global" statement refers to.
@@ -426,14 +367,15 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
            provider.isExpressionClassBody() and \
            closure_variable.getName() == "__class__":
             SyntaxErrors.raiseSyntaxError(
-                reason     = "cannot make __class__ global",
-                source_ref = source_ref
+                "cannot make __class__ global",
+                source_ref
             )
 
         provider.registerProvidedVariable(
             variable = closure_variable
         )
 
+    # Drop this, not really part of our tree.
     return None
 
 
@@ -443,7 +385,8 @@ def handleNonlocalDeclarationNode(provider, node, source_ref):
     parameter_provider = provider
 
     while parameter_provider.isExpressionGeneratorObjectBody() or \
-          parameter_provider.isExpressionCoroutineObjectBody():
+          parameter_provider.isExpressionCoroutineObjectBody() or \
+          parameter_provider.isExpressionAsyncgenObjectBody():
         parameter_provider = parameter_provider.getParentVariableProvider()
 
     if parameter_provider.isExpressionClassBody():
@@ -454,21 +397,18 @@ def handleNonlocalDeclarationNode(provider, node, source_ref):
     for variable_name in node.names:
         if variable_name in parameter_names:
             SyntaxErrors.raiseSyntaxError(
-                reason       = "name '%s' is parameter and nonlocal" % (
+                "name '%s' is parameter and nonlocal" % (
                     variable_name
                 ),
-                source_ref   = None
-                                 if Options.isFullCompat() and \
-                                 python_version < 340 else
-                               source_ref,
-                display_file = not Options.isFullCompat() or \
-                               python_version >= 340,
-                display_line = not Options.isFullCompat() or \
-                               python_version >= 340
+                source_ref.atColumnNumber(node.col_offset)
             )
 
-    provider.addNonlocalsDeclaration(node.names, source_ref)
+    provider.addNonlocalsDeclaration(
+        names      = node.names,
+        source_ref = source_ref.atColumnNumber(node.col_offset)
+    )
 
+    # Drop this, not really part of our tree.
     return None
 
 
@@ -510,22 +450,12 @@ def buildEllipsisNode(source_ref):
 def buildStatementLoopContinue(node, source_ref):
     # Python forbids this, although technically it's probably not much of
     # an issue.
+    source_ref = source_ref.atColumnNumber(node.col_offset)
+
     if getBuildContext() == "finally":
-        if not Options.isFullCompat() or python_version >= 300:
-            col_offset = node.col_offset - 9
-        else:
-            col_offset = None
-
-        if python_version >= 300 and Options.isFullCompat():
-            source_line = ""
-        else:
-            source_line = None
-
         SyntaxErrors.raiseSyntaxError(
             "'continue' not supported inside 'finally' clause",
-            source_ref,
-            col_offset  = col_offset,
-            source_line = source_line
+            source_ref
         )
 
     return StatementLoopContinue(
@@ -535,10 +465,10 @@ def buildStatementLoopContinue(node, source_ref):
 
 def buildStatementLoopBreak(provider, node, source_ref):
     # A bit unusual, we need the provider, but not the node,
-    # pylint: disable=W0613
+    # pylint: disable=unused-argument
 
     return StatementLoopBreak(
-        source_ref = source_ref
+        source_ref = source_ref.atColumnNumber(node.col_offset)
     )
 
 
@@ -554,12 +484,7 @@ def buildReturnNode(provider, node, source_ref):
     if provider.isExpressionClassBody() or provider.isCompiledPythonModule():
         SyntaxErrors.raiseSyntaxError(
             "'return' outside function",
-            source_ref,
-            None if python_version < 300 else (
-                node.col_offset
-                  if provider.isCompiledPythonModule() else
-                node.col_offset+4
-            )
+            source_ref.atColumnNumber(node.col_offset)
         )
 
     expression = buildNode(provider, node.value, source_ref, allow_none = True)
@@ -568,16 +493,25 @@ def buildReturnNode(provider, node, source_ref):
         if expression is not None and python_version < 330:
             SyntaxErrors.raiseSyntaxError(
                 "'return' with argument inside generator",
-                source_ref = source_ref,
+                source_ref.atColumnNumber(node.col_offset)
             )
 
+    if provider.isExpressionAsyncgenObjectBody():
+        if expression is not None:
+            SyntaxErrors.raiseSyntaxError(
+                "'return' with value in async generator",
+                source_ref.atColumnNumber(node.col_offset)
+            )
+
+
     if expression is None:
         expression = ExpressionConstantNoneRef(
             source_ref    = source_ref,
             user_provided = True
         )
 
-    if provider.isExpressionGeneratorObjectBody():
+    if provider.isExpressionGeneratorObjectBody() or \
+       provider.isExpressionAsyncgenObjectBody():
         return StatementGeneratorReturn(
             expression = expression,
             source_ref = source_ref
@@ -620,13 +554,14 @@ def buildUnaryOpNode(provider, node, source_ref):
 def buildBinaryOpNode(provider, node, source_ref):
     operator = getKind(node.op)
 
-    if operator == "Div" and source_ref.getFutureSpec().isFutureDivision():
-        operator = "TrueDiv"
+    if operator == "Div":
+        if getFutureSpec().isFutureDivision():
+            operator = "TrueDiv"
 
     left       = buildNode(provider, node.left, source_ref)
     right      = buildNode(provider, node.right, source_ref)
 
-    result = ExpressionOperationBinary(
+    result = makeBinaryOperationNode(
         operator   = operator,
         left       = left,
         right      = right,
@@ -664,10 +599,54 @@ def buildAwaitNode(provider, node, source_ref):
     )
 
 
+def buildFormattedValueNode(provider, node, source_ref):
+    value = buildNode(provider, node.value, source_ref)
+
+    conversion = node.conversion % 4 if node.conversion > 0 else 0
+
+    if conversion == 0:
+        pass
+    elif conversion == 3:
+        value = ExpressionBuiltinStr(
+            value      = value,
+            encoding   = None,
+            errors     = None,
+            source_ref = source_ref
+        )
+    elif conversion == 2:
+        value = ExpressionOperationUnary(
+            operator   = "Repr",
+            operand    = value,
+            source_ref = source_ref
+        )
+    elif conversion == 1:
+        value = ExpressionBuiltinAscii(
+            value      = value,
+            source_ref = source_ref
+        )
+    else:
+        assert False, conversion
+
+    return ExpressionBuiltinFormat(
+        value       = value,
+        format_spec = buildNode(provider, node.format_spec, source_ref, allow_none = True),
+        source_ref  = source_ref
+    )
+
+
+def buildJoinedStrNode(provider, node, source_ref):
+    return ExpressionStringConcatenation(
+        values     = buildNodeList(provider, node.values, source_ref),
+        source_ref = source_ref
+    )
+
+
+
 setBuildingDispatchers(
     path_args3 = {
         "Name"              : buildVariableReferenceNode,
         "Assign"            : buildAssignNode,
+        "AnnAssign"         : buildAnnAssignNode,
         "Delete"            : buildDeleteNode,
         "Lambda"            : buildLambdaNode,
         "GeneratorExp"      : buildGeneratorExpressionNode,
@@ -686,7 +665,7 @@ setBuildingDispatchers(
         "Global"            : handleGlobalDeclarationNode,
         "Nonlocal"          : handleNonlocalDeclarationNode,
         "TryExcept"         : buildTryExceptionNode,
-        "TryFinally"        : _buildTryFinallyNode,
+        "TryFinally"        : buildTryFinallyNode2,
         "Try"               : buildTryNode,
         "Raise"             : buildRaiseNode,
         "Import"            : buildImportModulesNode,
@@ -714,6 +693,8 @@ setBuildingDispatchers(
         "AugAssign"         : buildInplaceAssignNode,
         "IfExp"             : buildConditionalExpressionNode,
         "Break"             : buildStatementLoopBreak,
+        "JoinedStr"         : buildJoinedStrNode,
+        "FormattedValue"    : buildFormattedValueNode,
     },
     path_args2 = {
         "NameConstant" : buildNamedConstantNode,
@@ -730,7 +711,9 @@ setBuildingDispatchers(
 
 def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     # There are a bunch of branches here, mostly to deal with version
-    # differences for module default variables.
+    # differences for module default variables. pylint: disable=too-many-branches
+
+    pushFutureSpec()
 
     body = parseSourceCodeToAst(
         source_code = source_code,
@@ -739,6 +722,9 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     )
     body, doc = extractDocFromBody(body)
 
+    if is_module and is_main and python_version >= 360:
+        provider.markAsNeedsAnnotationsDictionary()
+
     result = buildStatementsNode(
         provider   = provider,
         nodes      = body,
@@ -758,10 +744,8 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
             for path_imported_name in getPthImportedPackages():
                 statements.append(
                     StatementExpressionOnly(
-                        expression = ExpressionImportModule(
+                        expression = makeAbsoluteImportNode(
                             module_name = path_imported_name,
-                            import_list = (),
-                            level       = 0,
                             source_ref  = source_ref,
                         ),
                         source_ref = source_ref
@@ -770,10 +754,8 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
 
             statements.append(
                 StatementExpressionOnly(
-                    expression = ExpressionImportModule(
+                    expression = makeAbsoluteImportNode(
                         module_name = "site",
-                        import_list = (),
-                        level       = 0,
                         source_ref  = source_ref,
                     ),
                     source_ref = source_ref
@@ -869,6 +851,24 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
             )
         )
 
+    if provider.needsAnnotationsDictionary():
+        # Set "__annotations__" on module level to {}
+        statements.append(
+            StatementAssignmentVariable(
+                variable_ref = ExpressionTargetVariableRef(
+                    variable_name = "__annotations__",
+                    source_ref    = internal_source_ref
+                ),
+                source       = makeConstantRefNode(
+                    constant      = {},
+                    source_ref    = internal_source_ref,
+                    user_provided = True
+                ),
+                source_ref   = internal_source_ref
+            )
+        )
+
+
     # Now the module body if there is any at all.
     if result is not None:
         statements.extend(
@@ -894,28 +894,32 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
 
 
     if is_module:
-        return makeModuleFrame(
+        result = makeModuleFrame(
             module     = provider,
             statements = statements,
             source_ref = source_ref
         )
+
+        provider.future_spec = popFutureSpec()
+
+        return result
     else:
         assert False
 
 
 def decideModuleTree(filename, package, is_shlib, is_top, is_main):
-    # Many variables, branches, due to the many cases, pylint: disable=R0912,R0915
+    # Many variables, branches, due to the many cases, pylint: disable=too-many-branches,too-many-statements
 
     assert package is None or type(package) is str
     assert filename is not None
 
-    if is_main and Utils.isDir(filename):
-        source_filename = Utils.joinpath(filename, "__main__.py")
+    if is_main and os.path.isdir(filename):
+        source_filename = os.path.join(filename, "__main__.py")
 
-        if not Utils.isFile(source_filename):
+        if not os.path.isfile(source_filename):
             sys.stderr.write(
                 "%s: can't find '__main__' module in '%s'\n" % (
-                    Utils.basename(sys.argv[0]),
+                    os.path.basename(sys.argv[0]),
                     filename
                 )
             )
@@ -927,7 +931,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
     else:
         main_added = False
 
-    if Utils.isFile(filename):
+    if os.path.isfile(filename):
         source_filename = filename
 
         source_ref = SourceCodeReferences.fromFilename(
@@ -937,7 +941,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
         if is_main:
             module_name = "__main__"
         else:
-            module_name = Utils.basename(filename)
+            module_name = os.path.basename(filename)
 
             if module_name.endswith(".py"):
                 module_name = module_name[:-3]
@@ -981,13 +985,13 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
 
     elif Importing.isPackageDir(filename):
         if is_top:
-            package_name = Utils.splitpath(filename)[-1]
+            package_name = splitPath(filename)[-1]
         else:
-            package_name = Utils.basename(filename)
+            package_name = os.path.basename(filename)
 
-        source_filename = Utils.joinpath(filename, "__init__.py")
+        source_filename = os.path.join(filename, "__init__.py")
 
-        if not Utils.isFile(source_filename):
+        if not os.path.isfile(source_filename):
             source_ref, result = createNamespacePackage(
                 package_name   = package_name,
                 module_relpath = filename
@@ -995,7 +999,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
             source_filename = None
         else:
             source_ref = SourceCodeReferences.fromFilename(
-                filename = Utils.abspath(source_filename),
+                filename = os.path.abspath(source_filename),
             )
 
             if package is not None:
@@ -1009,10 +1013,12 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
                 mode         = Plugins.decideCompilation(full_name, source_ref),
                 source_ref   = source_ref
             )
+
+            assert result.getFullName() == full_name, result
     else:
         sys.stderr.write(
             "%s: can't open file '%s'.\n" % (
-                Utils.basename(sys.argv[0]),
+                os.path.basename(sys.argv[0]),
                 filename
             )
         )
@@ -1069,7 +1075,7 @@ def createModuleTree(module, source_ref, source_code, is_main):
     if Options.isShowMemory():
         memory_watch.finish()
 
-        Tracing.printLine(
+        info(
             "Memory usage changed loading module '%s': %s" % (
                 module.getFullName(),
                 memory_watch.asStr()
@@ -1089,11 +1095,16 @@ def buildModuleTree(filename, package, is_top, is_main):
     # If there is source code associated (not the case for namespace packages of
     # Python3.3 or higher, then read it.
     if source_filename is not None:
+        source_code = readSourceCodeFromFilename(module.getFullName(), source_filename)
+
+        if is_main:
+            checkPythonVersionFromCode(source_code)
+
         # Read source code.
         createModuleTree(
             module      = module,
             source_ref  = source_ref,
-            source_code = readSourceCodeFromFilename(module.getFullName(), source_filename),
+            source_code = source_code,
             is_main     = is_main
         )
 
