@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -44,7 +44,7 @@ from nuitka.nodes.ComparisonNodes import ExpressionComparisonIsNOT
 from nuitka.nodes.ConditionalNodes import StatementConditional
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantEllipsisRef
 from nuitka.nodes.ContainerOperationNodes import ExpressionListOperationPop
-from nuitka.nodes.OperatorNodes import ExpressionOperationBinaryInplace
+from nuitka.nodes.OperatorNodes import makeExpressionOperationBinaryInplace
 from nuitka.nodes.SliceNodes import (
     ExpressionBuiltinSlice,
     ExpressionSliceLookup,
@@ -61,11 +61,14 @@ from nuitka.nodes.VariableRefNodes import (
     ExpressionVariableRef
 )
 from nuitka.PythonVersions import python_version
+from nuitka.tree.ReformulationImportStatements import getFutureSpec
 
 from .Helpers import (
     buildNode,
     getKind,
+    makeConstantRefNode,
     makeSequenceCreationOrConstant,
+    makeStatementsSequence,
     makeStatementsSequenceFromStatement,
     makeStatementsSequenceFromStatements,
     makeStatementsSequenceOrStatement,
@@ -119,7 +122,7 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                                          source_ref):
     # This is using many variable names on purpose, so as to give names to the
     # unpacked detail values, and has many branches due to the many cases
-    # dealt with, pylint: disable=R0912,R0914
+    # dealt with, pylint: disable=too-many-branches,too-many-locals
 
     if kind == "Name":
         variable_ref = detail
@@ -184,20 +187,6 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
             name       = "source_iter"
         )
 
-        statements = [
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = source_iter_var,
-                    source_ref = source_ref
-                ),
-                source       = ExpressionBuiltinIter1(
-                    value      = source,
-                    source_ref = source_ref
-                ),
-                source_ref   = source_ref
-            )
-        ]
-
         element_vars = [
             provider.allocateTempVariable(
                 temp_scope = temp_scope,
@@ -212,19 +201,20 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
         starred_list_var = None
         starred_index = None
 
+        statements = []
+
         for element_index, element in enumerate(detail):
             element_var = element_vars[element_index]
 
             if starred_list_var is not None:
                 if element[0] == "Starred":
                     raiseSyntaxError(
-                        reason     = "two starred expressions in assignment",
-                        col_offset = 0,
-                        source_ref = source_ref
+                        "two starred expressions in assignment",
+                        source_ref.atColumnNumber(0)
                     )
 
                 statements.insert(
-                    starred_index+2,
+                    starred_index+1,
                     StatementAssignmentVariable(
                         variable_ref = ExpressionTargetTempVariableRef(
                             variable   = element_var,
@@ -292,6 +282,32 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                 )
             )
 
+
+        statements = [
+            StatementAssignmentVariable(
+                variable_ref = ExpressionTargetTempVariableRef(
+                    variable   = source_iter_var,
+                    source_ref = source_ref
+                ),
+                source       = ExpressionBuiltinIter1(
+                    value      = source,
+                    source_ref = source_ref
+                ),
+                source_ref   = source_ref
+            ),
+            makeTryFinallyStatement(
+                provider   = provider,
+                tried      = statements,
+                final      = (
+                    StatementReleaseVariable(
+                        variable   = source_iter_var,
+                        source_ref = source_ref
+                    ),
+                ),
+                source_ref = source_ref
+            )
+        ]
+
         # When all is done, copy over to the actual assignment targets, starred
         # or not makes no difference here anymore.
         for element_index, element in enumerate(detail):
@@ -313,21 +329,26 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
                 )
             )
 
+            # Need to release temporary variables right after successful
+            # usage.
+            statements.append(
+                StatementDelVariable(
+                    variable_ref = ExpressionTargetTempVariableRef(
+                        variable   = element_var,
+                        source_ref = source_ref
+                    ),
+                    tolerant     = True,
+                    source_ref   = source_ref,
+                )
+            )
+
         final_statements = []
 
-        final_statements.append(
-            StatementReleaseVariable(
-                variable   = source_iter_var,
-                source_ref = source_ref
-            )
-        )
-
-        # TODO: In that order, or reversed.
         for element_var in element_vars:
             final_statements.append(
                 StatementReleaseVariable(
                     variable   = element_var,
-                    source_ref = source_ref
+                    source_ref = source_ref,
                 )
             )
 
@@ -339,9 +360,8 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source,
         )
     elif kind == "Starred":
         raiseSyntaxError(
-            reason     = "starred assignment target must be in a list or tuple",
-            source_ref = source_ref,
-            col_offset = 0
+            "starred assignment target must be in a list or tuple",
+            source_ref.atColumnNumber(0)
         )
     else:
         assert False, (kind, source_ref, detail)
@@ -372,7 +392,7 @@ def buildAssignmentStatements(provider, node, source, source_ref,
 
 def decodeAssignTarget(provider, node, source_ref, allow_none = False):
     # Many cases to deal with, because of the different assign targets,
-    # pylint: disable=R0911,R0912
+    # pylint: disable=too-many-branches,too-many-return-statements
 
     if node is None and allow_none:
         return None
@@ -527,7 +547,80 @@ def buildAssignNode(provider, node, source_ref):
         )
 
 
-def buildDeleteStatementFromDecoded(kind, detail, source_ref):
+def buildAnnAssignNode(provider, node, source_ref):
+    """ Python3.6 annotation assignment.
+
+    """
+
+    if provider.isCompiledPythonModule() or provider.isExpressionClassBody():
+        provider.markAsNeedsAnnotationsDictionary()
+
+    # Evaluate the right hand side first, so it can get names provided
+    # before the left hand side exists.
+    statements = []
+
+    if node.value is not None:
+        source = buildNode(provider, node.value, source_ref)
+
+        statements.append(
+            buildAssignmentStatements(
+                provider   = provider,
+                node       = node.target,
+                source     = source,
+                source_ref = source_ref
+            )
+        )
+
+        # Only name referencing annotations are effective right now.
+        if statements[-1].isStatementAssignmentVariable():
+            variable_name = statements[-1].getTargetVariableRef().getVariableName()
+        else:
+            variable_name = None
+    else:
+        # Only name referencing annotations are effective right now.
+        kind, detail = decodeAssignTarget(
+            provider   = provider,
+            node       = node.target,
+            source_ref = source_ref
+        )
+
+        if kind == "Name":
+            variable_name = detail.getVariableName()
+        else:
+            variable_name = None
+
+    # Only annotations for modules and classes are really made, for functions
+    # they are ignored like comments.
+    if variable_name is not None:
+        if provider.isExpressionFunctionBody():
+            provider.getVariableForAssignment(variable_name)
+        else:
+            annotation = buildNode(provider, node.annotation, source_ref)
+
+            statements.append(
+                StatementAssignmentSubscript(
+                    expression = ExpressionVariableRef(
+                        variable_name = "__annotations__",
+                        variable      = provider.getVariableForAssignment("__annotations__"),
+                        source_ref    = source_ref
+                    ),
+                    subscript  = makeConstantRefNode(
+                        constant   = variable_name,
+                        source_ref = source_ref
+                    ),
+                    source     = annotation,
+                    source_ref = source_ref
+                )
+            )
+
+    return makeStatementsSequence(
+        statements = statements,
+        allow_none = True,
+        source_ref = source_ref
+    )
+
+
+def buildDeleteStatementFromDecoded(node, kind, detail, source_ref):
     if kind in ("Name", "Name_Exception"):
         # Note: Name_Exception is a "del" for exception handlers that doesn't
         # insist on the variable being defined, user code may do it too, and
@@ -537,7 +630,7 @@ def buildDeleteStatementFromDecoded(kind, detail, source_ref):
         return StatementDelVariable(
             variable_ref = variable_ref,
             tolerant     = kind == "Name_Exception",
-            source_ref   = source_ref
+            source_ref   = source_ref.atColumnNumber(node.col_offset+1)
         )
     elif kind == "Attribute":
         lookup_source, attribute_name = detail
@@ -584,6 +677,7 @@ def buildDeleteStatementFromDecoded(kind, detail, source_ref):
         for sub_node in detail:
             result.append(
                 buildDeleteStatementFromDecoded(
+                    node       = node,
                     kind       = sub_node[0],
                     detail     = sub_node[1],
                     source_ref = source_ref
@@ -617,6 +711,7 @@ def buildDeleteNode(provider, node, source_ref):
 
         statements.append(
             buildDeleteStatementFromDecoded(
+                node       = target,
                 kind       = kind,
                 detail     = detail,
                 source_ref = source_ref
@@ -633,7 +728,7 @@ def _buildInplaceAssignVariableNode(variable_ref, operator, expression,
                                     source_ref):
     assert variable_ref.isExpressionTargetVariableRef(), variable_ref
 
-    inplace_node = ExpressionOperationBinaryInplace(
+    inplace_node = makeExpressionOperationBinaryInplace(
         operator   = operator,
         left       = ExpressionVariableRef(
             variable_name = variable_ref.getVariableName(),
@@ -682,7 +777,7 @@ def _buildInplaceAssignAttributeNode(provider, lookup_source, attribute_name,
             variable   = tmp_variable2,
             source_ref = source_ref
         ),
-        source       = ExpressionOperationBinaryInplace(
+        source       = makeExpressionOperationBinaryInplace(
             operator   = operator,
             left       = ExpressionTempVariableRef(
                 variable   = tmp_variable1,
@@ -781,7 +876,7 @@ def _buildInplaceAssignSubscriptNode(provider, subscribed, subscript,
             variable   = tmp_variable2,
             source_ref = source_ref
         ),
-        source     = ExpressionOperationBinaryInplace(
+        source     = makeExpressionOperationBinaryInplace(
             operator   = operator,
             left       = ExpressionSubscriptLookup(
                 subscribed = ExpressionTempVariableRef(
@@ -831,7 +926,7 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
 
     # Due to the 3 inputs, which we need to also put into temporary variables,
     # there are too many variables here, but they are needed.
-    # pylint: disable=R0914
+    # pylint: disable=too-many-locals
 
     # First assign the target value, lower and upper to temporary variables.
     copy_to_tmp = StatementAssignmentVariable(
@@ -929,7 +1024,7 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
                     step       = None,
                     source_ref = source_ref
                 ),
-                source     = ExpressionOperationBinaryInplace(
+                source     = makeExpressionOperationBinaryInplace(
                     operator   = operator,
                     left       = ExpressionSubscriptLookup(
                         subscribed = ExpressionTempVariableRef(
@@ -959,7 +1054,7 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
                 ),
                 lower      = lower_ref1,
                 upper      = upper_ref1,
-                source     = ExpressionOperationBinaryInplace(
+                source     = makeExpressionOperationBinaryInplace(
                     operator   = operator,
                     left       = ExpressionSliceLookup(
                         expression = ExpressionTempVariableRef(
@@ -991,11 +1086,11 @@ def _buildInplaceAssignSliceNode(provider, lookup_source, lower, upper,
 def buildInplaceAssignNode(provider, node, source_ref):
     # There are many inplace assignment variables, and the detail is unpacked
     # into names, so we end up with a lot of variables, which is on purpose,
-    # pylint: disable=R0914
+    # pylint: disable=too-many-locals
 
     operator = getKind(node.op)
 
-    if operator == "Div" and source_ref.getFutureSpec().isFutureDivision():
+    if operator == "Div" and getFutureSpec().isFutureDivision():
         operator = "TrueDiv"
 
     operator = 'I' + operator
