@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -44,7 +44,7 @@ from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
 from nuitka.nodes.DictionaryNodes import (
     ExpressionDictOperationGet,
@@ -70,9 +70,9 @@ from nuitka.nodes.VariableRefNodes import (
 from nuitka.PythonVersions import python_version
 
 from .Helpers import (
+    buildFrameNode,
     buildNode,
     buildNodeList,
-    buildStatementsNode,
     extractDocFromBody,
     getKind,
     makeDictCreationOrConstant,
@@ -85,7 +85,7 @@ from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 def _buildClassNode3(provider, node, source_ref):
     # Many variables, due to the huge re-formulation that is going on here,
-    # which just has the complexity, pylint: disable=R0914
+    # which just has the complexity, pylint: disable=too-many-locals
 
     # This function is the Python3 special case with special re-formulation as
     # according to developer manual.
@@ -122,46 +122,32 @@ def _buildClassNode3(provider, node, source_ref):
         source_ref = source_ref
     )
 
-    if python_version >= 340 and False: # TODO: Temporarily reverted:
-        tmp_class = class_creation_function.allocateTempVariable(
-            temp_scope = None,
-            name       = "__class__"
-        )
-
-        class_target_variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_class,
-            source_ref = source_ref
-        )
-        class_variable_ref = ExpressionTempVariableRef(
-            variable   = tmp_class,
-            source_ref = source_ref
-        )
-    else:
-        class_variable = class_creation_function.getVariableForAssignment(
-            "__class__"
-        )
-
-        class_target_variable_ref = ExpressionTargetVariableRef(
-            variable_name = "__class__",
-            variable      = class_variable,
-            source_ref    = source_ref
-        )
-        class_variable_ref = ExpressionVariableRef(
-            variable_name = "__class__",
-            variable      = class_variable,
-            source_ref    = source_ref
-        )
-
-    code_object = CodeObjectSpec(
-        code_name     = node.name,
-        code_kind     = "Class",
-        arg_names     = (),
-        kw_only_count = 0,
-        has_starlist  = False,
-        has_stardict  = False
+    class_variable = class_creation_function.getVariableForAssignment(
+        "__class__"
     )
 
-    body = buildStatementsNode(
+    class_target_variable_ref = ExpressionTargetVariableRef(
+        variable_name = "__class__",
+        variable      = class_variable,
+        source_ref    = source_ref
+    )
+    class_variable_ref = ExpressionVariableRef(
+        variable_name = "__class__",
+        variable      = class_variable,
+        source_ref    = source_ref
+    )
+
+    code_object = CodeObjectSpec(
+        co_name           = node.name,
+        co_kind           = "Class",
+        co_varnames       = (),
+        co_argcount       = 0,
+        co_kwonlyargcount = 0,
+        co_has_starlist   = False,
+        co_has_stardict   = False
+    )
+
+    body = buildFrameNode(
         provider    = class_creation_function,
         nodes       = class_statement_nodes,
         code_object = code_object,
@@ -192,7 +178,7 @@ def _buildClassNode3(provider, node, source_ref):
                 variable      = module_variable,
                 source_ref    = source_ref
             ),
-            source       = ExpressionConstantRef(
+            source       = makeConstantRefNode(
                 constant      = provider.getParentModule().getFullName(),
                 source_ref    = source_ref,
                 user_provided = True
@@ -213,7 +199,7 @@ def _buildClassNode3(provider, node, source_ref):
                     variable      = doc_variable,
                     source_ref    = source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = class_doc,
                     source_ref    = source_ref,
                     user_provided = True
@@ -230,7 +216,7 @@ def _buildClassNode3(provider, node, source_ref):
         )
 
         if python_version < 340:
-            qualname_ref = ExpressionConstantRef(
+            qualname_ref = makeConstantRefNode(
                 constant      = qualname,
                 source_ref    = source_ref,
                 user_provided = True
@@ -256,8 +242,31 @@ def _buildClassNode3(provider, node, source_ref):
         if python_version >= 340:
             qualname_assign = statements[-1]
 
+    if python_version >= 360 and \
+       class_creation_function.needsAnnotationsDictionary():
+        annotations_variable = class_creation_function.getVariableForAssignment(
+            "__annotations__"
+        )
+
+        statements.append(
+            StatementAssignmentVariable(
+                variable_ref = ExpressionTargetVariableRef(
+                    variable_name = "__annotations__",
+                    variable      = annotations_variable,
+                    source_ref    = source_ref
+                ),
+                source       = makeConstantRefNode(
+                    constant      = {},
+                    source_ref    = source_ref,
+                    user_provided = True
+                ),
+                source_ref   = source_ref
+            )
+        )
+
+    statements.append(body)
+
     statements += [
-        body,
         StatementAssignmentVariable(
             variable_ref = class_target_variable_ref,
             source       = ExpressionCall(
@@ -268,7 +277,7 @@ def _buildClassNode3(provider, node, source_ref):
                 args       = makeSequenceCreationOrConstant(
                     sequence_kind = "tuple",
                     elements      = (
-                        ExpressionConstantRef(
+                        makeConstantRefNode(
                             constant      = node.name,
                             source_ref    = source_ref,
                             user_provided = True
@@ -371,7 +380,7 @@ def _buildClassNode3(provider, node, source_ref):
             ),
             source       = makeDictCreationOrConstant(
                 keys       = [
-                    ExpressionConstantRef(
+                    makeConstantRefNode(
                         constant      = keyword.arg,
                         source_ref    = source_ref,
                         user_provided = True
@@ -396,7 +405,7 @@ def _buildClassNode3(provider, node, source_ref):
             source       = ExpressionSelectMetaclass(
                 metaclass  = ExpressionConditional(
                     condition      = ExpressionComparisonIn(
-                        left       = ExpressionConstantRef(
+                        left       = makeConstantRefNode(
                             constant      = "metaclass",
                             source_ref    = source_ref,
                             user_provided = True
@@ -412,7 +421,7 @@ def _buildClassNode3(provider, node, source_ref):
                             variable   = tmp_class_decl_dict,
                             source_ref = source_ref
                         ),
-                        key        = ExpressionConstantRef(
+                        key        = makeConstantRefNode(
                             constant      = "metaclass",
                             source_ref    = source_ref,
                             user_provided = True
@@ -434,7 +443,7 @@ def _buildClassNode3(provider, node, source_ref):
                                     variable   = tmp_bases,
                                     source_ref = source_ref
                                 ),
-                                subscript  = ExpressionConstantRef(
+                                subscript  = makeConstantRefNode(
                                     constant      = 0,
                                     source_ref    = source_ref,
                                     user_provided = True
@@ -457,7 +466,7 @@ def _buildClassNode3(provider, node, source_ref):
         ),
         StatementConditional(
             condition  = ExpressionComparisonIn(
-                left       = ExpressionConstantRef(
+                left       = makeConstantRefNode(
                     constant      = "metaclass",
                     source_ref    = source_ref,
                     user_provided = True
@@ -475,7 +484,7 @@ def _buildClassNode3(provider, node, source_ref):
                         variable   = tmp_class_decl_dict,
                         source_ref = source_ref
                     ),
-                    key        = ExpressionConstantRef(
+                    key        = makeConstantRefNode(
                         constant      = "metaclass",
                         source_ref    = source_ref,
                         user_provided = True
@@ -496,14 +505,14 @@ def _buildClassNode3(provider, node, source_ref):
                         variable   = tmp_metaclass,
                         source_ref = source_ref
                     ),
-                    name       = ExpressionConstantRef(
+                    name       = makeConstantRefNode(
                         constant      = "__prepare__",
                         source_ref    = source_ref,
                         user_provided = True
                     ),
                     source_ref = source_ref
                 ),
-                expression_no  = ExpressionConstantRef(
+                expression_no  = makeConstantRefNode(
                     constant      = {},
                     source_ref    = source_ref,
                     user_provided = True
@@ -519,7 +528,7 @@ def _buildClassNode3(provider, node, source_ref):
                     ),
                     args       = ExpressionMakeTuple(
                         elements   = (
-                            ExpressionConstantRef(
+                            makeConstantRefNode(
                                 constant      = node.name,
                                 source_ref    = source_ref,
                                 user_provided = True
@@ -585,7 +594,7 @@ def _buildClassNode3(provider, node, source_ref):
 
 def _buildClassNode2(provider, node, source_ref):
     # This function is the Python2 special case with special re-formulation as
-    # according to developer manual, and it's very detailed, pylint: disable=R0914
+    # according to developer manual, and it's very detailed, pylint: disable=too-many-locals
     class_statement_nodes, class_doc = extractDocFromBody(node)
 
     function_body = ExpressionClassBody(
@@ -597,15 +606,16 @@ def _buildClassNode2(provider, node, source_ref):
     )
 
     code_object = CodeObjectSpec(
-        code_name     = node.name,
-        code_kind     = "Class",
-        arg_names     = (),
-        kw_only_count = 0,
-        has_starlist  = False,
-        has_stardict  = False
+        co_name           = node.name,
+        co_kind           = "Class",
+        co_varnames       = (),
+        co_argcount       = 0,
+        co_kwonlyargcount = 0,
+        co_has_starlist   = False,
+        co_has_stardict   = False
     )
 
-    body = buildStatementsNode(
+    body = buildFrameNode(
         provider    = function_body,
         nodes       = class_statement_nodes,
         code_object = code_object,
@@ -624,7 +634,7 @@ def _buildClassNode2(provider, node, source_ref):
                 variable_name = "__module__",
                 source_ref    = source_ref
             ),
-            source       = ExpressionConstantRef(
+            source       = makeConstantRefNode(
                 constant      = provider.getParentModule().getFullName(),
                 source_ref    = source_ref,
                 user_provided = True
@@ -640,7 +650,7 @@ def _buildClassNode2(provider, node, source_ref):
                     variable_name = "__doc__",
                     source_ref    = source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = class_doc,
                     source_ref    = source_ref,
                     user_provided = True
@@ -721,7 +731,7 @@ def _buildClassNode2(provider, node, source_ref):
             ),
             source       = ExpressionConditional(
                 condition      =  ExpressionComparisonIn(
-                    left       = ExpressionConstantRef(
+                    left       = makeConstantRefNode(
                         constant      = "__metaclass__",
                         source_ref    = source_ref,
                         user_provided = True
@@ -737,7 +747,7 @@ def _buildClassNode2(provider, node, source_ref):
                         variable   = tmp_class_dict,
                         source_ref = source_ref
                     ),
-                    key        = ExpressionConstantRef(
+                    key        = makeConstantRefNode(
                         constant      = "__metaclass__",
                         source_ref    = source_ref,
                         user_provided = True
@@ -768,7 +778,7 @@ def _buildClassNode2(provider, node, source_ref):
                 ),
                 args       = ExpressionMakeTuple(
                     elements   = (
-                        ExpressionConstantRef(
+                        makeConstantRefNode(
                             constant      = node.name,
                             source_ref    = source_ref,
                             user_provided = True
