@@ -52,25 +52,26 @@ from nuitka.nodes.StatementNodes import (
     StatementsSequence
 )
 from nuitka.PythonVersions import python_version
-from nuitka.tree.Helpers import makeReraiseExceptionStatement
 
 from .Helpers import (
     buildNode,
     buildStatementsNode,
     getKind,
     makeConditionalStatement,
+    makeReraiseExceptionStatement,
     makeStatementsSequence,
     makeStatementsSequenceFromStatement
 )
 from .ReformulationAssignmentStatements import buildAssignmentStatements
-from .ReformulationTryExceptStatements import \
+from .ReformulationTryExceptStatements import (
     makeTryExceptSingleHandlerNodeWithPublish
+)
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                    sync, source_ref):
-    # Many details, pylint: disable=R0914
+    # Many details, pylint: disable=too-many-locals
     with_source = buildNode(provider, context_expr, source_ref)
 
     if Options.isFullCompat():
@@ -360,7 +361,7 @@ def buildWithNode(provider, node, source_ref):
     # The body for the first context manager is the other things.
     body = buildStatementsNode(provider, node.body, source_ref)
 
-    assert len(context_exprs) > 0 and len(context_exprs) == len(assign_targets)
+    assert context_exprs and len(context_exprs) == len(assign_targets)
 
     context_exprs.reverse()
     assign_targets.reverse()
@@ -399,7 +400,7 @@ def buildAsyncWithNode(provider, node, source_ref):
     # The body for the first context manager is the other things.
     body = buildStatementsNode(provider, node.body, source_ref)
 
-    assert len(context_exprs) > 0 and len(context_exprs) == len(assign_targets)
+    assert context_exprs and len(context_exprs) == len(assign_targets)
 
     context_exprs.reverse()
     assign_targets.reverse()
