@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,7 +32,7 @@ def getOwnProcessMemoryUsage():
         import ctypes.wintypes
 
         # Lets allow this to match Windows API it reflects,
-        # pylint: disable=C0103
+        # pylint: disable=invalid-name
         class PROCESS_MEMORY_COUNTERS_EX(ctypes.Structure):
             _fields_ = [
                 ("cb", ctypes.wintypes.DWORD),
@@ -68,7 +68,7 @@ def getOwnProcessMemoryUsage():
 
         return counters.PrivateUsage
     else:
-        # Posix only code, pylint: disable=F0401,I0021
+        # Posix only code, pylint: disable=import-error,useless-suppression
         import resource  # @UnresolvedImport
 
         # The value is from "getrusage", which has OS dependent scaling, at least
@@ -104,7 +104,7 @@ def getHumanReadableProcessMemoryUsage(value = None):
         return "%d bytes" % value
 
 
-class MemoryWatch:
+class MemoryWatch(object):
     def __init__(self):
         self.start = getOwnProcessMemoryUsage()
         self.stop = None
